package com.example.echo.client;

import com.google.gwt.core.client.GWT;
import com.google.gwt.dom.client.Element;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HTMLPanel;
import com.google.gwt.user.client.ui.Widget;

public class BootstrapWidgets extends Composite {

    interface BootstrapWidgetsUiBinder extends UiBinder<Widget, BootstrapWidgets> {
    }

    private static BootstrapWidgetsUiBinder uiBinder = GWT.create(BootstrapWidgetsUiBinder.class);

    @UiField HTMLPanel modalContainer;
    @UiField Button tooltipButton;
    @UiField Button popoverButton;
    @UiField HTMLPanel carouselContainer;
    @UiField Button primaryButton;
    @UiField Button secondaryButton;
    @UiField Button successButton;
    @UiField HTMLPanel alertContainer;
    @UiField HTMLPanel cardContainer;
    @UiField Button cardButton;
    @UiField Button launchModalButton;
    @UiField HTMLPanel dropdownContainer;
    @UiField HTMLPanel tabsContainer;
    @UiField Button homeTabButton;
    @UiField Button profileTabButton;

    public BootstrapWidgets() {
        initWidget(uiBinder.createAndBindUi(this));

        // Add click handlers for the buttons
        primaryButton.addClickHandler(new ClickHandler() {
            @Override
            public void onClick(ClickEvent event) {
                Window.alert("Primary button clicked!");
            }
        });

        secondaryButton.addClickHandler(new ClickHandler() {
            @Override
            public void onClick(ClickEvent event) {
                Window.alert("Secondary button clicked!");
            }
        });

        successButton.addClickHandler(new ClickHandler() {
            @Override
            public void onClick(ClickEvent event) {
                Window.alert("Success button clicked!");
            }
        });

        cardButton.addClickHandler(new ClickHandler() {
            @Override
            public void onClick(ClickEvent event) {
                Window.alert("Card button clicked!");
            }
        });

        // Set data attributes for the modal trigger button and handle click
        launchModalButton.getElement().setAttribute("data-bs-toggle", "modal");
        launchModalButton.getElement().setAttribute("data-bs-target", "#exampleModal");
        launchModalButton.addClickHandler(new ClickHandler() {
            @Override
            public void onClick(ClickEvent event) {
                // Optional: Add custom logic before showing the modal
                Window.alert("Launching modal...");
            }
        });

        // Set data attributes for tooltip
        tooltipButton.getElement().setAttribute("data-bs-toggle", "tooltip");
        tooltipButton.getElement().setAttribute("data-bs-placement", "top");
        tooltipButton.getElement().setAttribute("title", "This is a tooltip!");

        // Set data attributes for popover
        popoverButton.getElement().setAttribute("data-bs-toggle", "popover");
        popoverButton.getElement().setAttribute("data-bs-placement", "top");
        popoverButton.getElement().setAttribute("data-bs-content", "This is a popover content!");
        popoverButton.getElement().setAttribute("title", "Popover Title");

        // Set data attributes for tabs and handle clicks
        homeTabButton.getElement().setAttribute("id", "home-tab");
        homeTabButton.getElement().setAttribute("data-bs-toggle", "tab");
        homeTabButton.getElement().setAttribute("data-bs-target", "#home");
        homeTabButton.getElement().setAttribute("role", "tab");
        homeTabButton.getElement().setAttribute("aria-controls", "home");
        homeTabButton.getElement().setAttribute("aria-selected", "true");
        homeTabButton.addClickHandler(new ClickHandler() {
            @Override
            public void onClick(ClickEvent event) {
                Window.alert("Home tab clicked!");
            }
        });

        profileTabButton.getElement().setAttribute("id", "profile-tab");
        profileTabButton.getElement().setAttribute("data-bs-toggle", "tab");
        profileTabButton.getElement().setAttribute("data-bs-target", "#profile");
        profileTabButton.getElement().setAttribute("role", "tab");
        profileTabButton.getElement().setAttribute("aria-controls", "profile");
        profileTabButton.getElement().setAttribute("aria-selected", "false");
        profileTabButton.addClickHandler(new ClickHandler() {
            @Override
            public void onClick(ClickEvent event) {
                Window.alert("Profile tab clicked!");
            }
        });

        // Initialize Bootstrap components (tooltips, popovers, carousel, and tabs)
        initBootstrapComponents(tooltipButton.getElement(), popoverButton.getElement(), carouselContainer.getElement(), tabsContainer.getElement());
    }

    private native void initBootstrapComponents(Element tooltipElement, Element popoverElement, Element carouselElement, Element tabsElement) /*-{
        // Initialize tooltips (Bootstrap 5 syntax)
        new $wnd.bootstrap.Tooltip(tooltipElement);

        // Initialize popovers (Bootstrap 5 syntax)
        new $wnd.bootstrap.Popover(popoverElement);

        // Initialize carousel (Bootstrap 5 syntax)
        new $wnd.bootstrap.Carousel($wnd.jQuery(carouselElement).find('#carouselExample')[0], {
            interval: 5000 // Auto-slide every 5 seconds
        });

        // Initialize tabs (Bootstrap 5 syntax)
        $wnd.jQuery(tabsElement).find('[data-bs-toggle="tab"]').each(function() {
            new $wnd.bootstrap.Tab(this);
        });
    }-*/;
}
