package com.example.echo.client;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.dom.client.Document;
import com.google.gwt.dom.client.Element;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.Widget;

public class EchoClient implements EntryPoint {

    private FlowPanel widgetContainer;
    private EchoWidget echoWidget;
    private AllWidgets allWidgets;
    private BootstrapWidgets bootstrapWidgets;
    private LightboxGalleryWidget lightboxGalleryWidget;
    private Button showEchoButton;
    private Button showAllWidgetsButton;
    private Button showBootstrapWidgetsButton;
    private Button showLightboxGalleryButton;

    @Override
    public void onModuleLoad() {
        widgetContainer = new FlowPanel();
        widgetContainer.getElement().setId("widgetContainer");

        // Create buttons to switch widgets
        showEchoButton = new Button("Show Echo Widget");
        showEchoButton.setStyleName("btn btn-primary");
        showAllWidgetsButton = new Button("Show All Widgets");
        showAllWidgetsButton.setStyleName("btn btn-primary");
        showBootstrapWidgetsButton = new Button("Show Bootstrap Widgets");
        showBootstrapWidgetsButton.setStyleName("btn btn-primary");
        showLightboxGalleryButton = new Button("Show Lightbox Gallery");
        showLightboxGalleryButton.setStyleName("btn btn-primary");

        // Add click handlers
        showEchoButton.addClickHandler(new ClickHandler() {
            @Override
            public void onClick(ClickEvent event) {
                updateUrlAndView("/echo");
            }
        });

        showAllWidgetsButton.addClickHandler(new ClickHandler() {
            @Override
            public void onClick(ClickEvent event) {
                updateUrlAndView("/allWidgets");
            }
        });

        showBootstrapWidgetsButton.addClickHandler(new ClickHandler() {
            @Override
            public void onClick(ClickEvent event) {
                updateUrlAndView("/bootstrapWidgets");
            }
        });

        showLightboxGalleryButton.addClickHandler(new ClickHandler() {
            @Override
            public void onClick(ClickEvent event) {
                updateUrlAndView("/lightboxGallery");
            }
        });

        // Listen for popstate events (back/forward navigation)
        addPopStateListener();

        // Create a FlowPanel for buttons with flexbox styling for centering and spacing
        FlowPanel buttonPanel = new FlowPanel();
        buttonPanel.setStyleName("container mt-3 d-flex justify-content-center");
        buttonPanel.getElement().getStyle().setProperty("gap", "8px");
        buttonPanel.add(showEchoButton);
        buttonPanel.add(showAllWidgetsButton);
        buttonPanel.add(showBootstrapWidgetsButton);
        buttonPanel.add(showLightboxGalleryButton);

        // Add button panel and widget container to root
        RootPanel root = RootPanel.get("root");
        root.add(buttonPanel);
        root.add(widgetContainer);

        // Initialize the view based on the current URL
        initializeViewFromUrl();
    }

    private void updateUrlAndView(String path) {
        pushState(path);
        updateView(path);
    }

    private void initializeViewFromUrl() {
        String path = Window.Location.getPath();
        String token = path.substring(path.lastIndexOf("/") + 1);
        if (token == null || token.isEmpty() || path.equals("/GWTTest/") || path.equals("/GWTTest")) {
            token = "echo"; // Default view
            pushState("/echo");
        }
        updateView("/" + token);
    }

    private void updateView(String path) {
        String token = path.startsWith("/") ? path.substring(1) : path;
        Widget widget;
        switch (token) {
            case "echo":
                widget = getEchoWidget();
                break;
            case "allWidgets":
                widget = getAllWidgets();
                break;
            case "bootstrapWidgets":
                widget = getBootstrapWidgets();
                break;
            case "lightboxGallery":
                widget = getLightboxGalleryWidget();
                break;
            default:
                widget = getEchoWidget();
                pushState("/echo");
        }
        showWidget(widget);
    }

    private void showWidget(Widget widget) {
        widgetContainer.clear();
        widgetContainer.add(widget);

        // Update button states
        showEchoButton.setEnabled(widget != getEchoWidget());
        showAllWidgetsButton.setEnabled(widget != getAllWidgets());
        showBootstrapWidgetsButton.setEnabled(widget != getBootstrapWidgets());
        showLightboxGalleryButton.setEnabled(widget != getLightboxGalleryWidget());

        // Update metadata for SEO
        String viewName = widget == getEchoWidget() ? "Echo Widget"
                : widget == getAllWidgets() ? "All Widgets"
                        : widget == getBootstrapWidgets() ? "Bootstrap Widgets"
                                : "Lightbox Gallery";
        updateMetadata(viewName);
    }

    private void updateMetadata(String view) {
        Document.get().setTitle("GWT Echo Example - " + view);
        Element oldMeta = Document.get().getElementById("descriptionMeta");
        if (oldMeta != null) {
            oldMeta.removeFromParent();
        }
        Element meta = Document.get().createElement("meta");
        meta.setAttribute("id", "descriptionMeta");
        meta.setAttribute("name", "description");
        meta.setAttribute("content", "View the " + view + " page in the GWT Echo Example.");
        Document.get().getHead().appendChild(meta);

        Element oldScript = Document.get().getElementById("structuredData");
        if (oldScript != null) {
            oldScript.removeFromParent();
        }
        Element script = Document.get().createElement("script");
        script.setAttribute("id", "structuredData");
        script.setAttribute("type", "application/ld+json");
        script.setInnerHTML("{"
                + "\"@context\": \"https://schema.org\","
                + "\"@type\": \"WebPage\","
                + "\"name\": \"GWT Echo Example - " + view + "\","
                + "\"url\": \"" + Window.Location.getPath() + "\","
                + "\"description\": \"View the " + view + " page in the GWT Echo Example.\""
                + "}");
        Document.get().getHead().appendChild(script);
    }

    private EchoWidget getEchoWidget() {
        if (echoWidget == null) {
            echoWidget = new EchoWidget();
        }
        return echoWidget;
    }

    private AllWidgets getAllWidgets() {
        if (allWidgets == null) {
            allWidgets = new AllWidgets();
        }
        return allWidgets;
    }

    private BootstrapWidgets getBootstrapWidgets() {
        if (bootstrapWidgets == null) {
            bootstrapWidgets = new BootstrapWidgets();
        }
        return bootstrapWidgets;
    }

    private LightboxGalleryWidget getLightboxGalleryWidget() {
        if (lightboxGalleryWidget == null) {
            lightboxGalleryWidget = new LightboxGalleryWidget();
        }
        return lightboxGalleryWidget;
    }

    private native void pushState(String path) /*-{
        var fullPath = "/GWTTest" + path;
        $wnd.history.pushState({path: fullPath}, "", fullPath);
    }-*/;

    private native void addPopStateListener() /*-{
        var that = this;
        $wnd.addEventListener("popstate", function(event) {
            if (event.state && event.state.path) {
                that.@com.example.echo.client.EchoClient::updateView(Ljava/lang/String;)(event.state.path.replace("/GWTTest", ""));
            } else {
                var path = $wnd.location.pathname.replace("/GWTTest", "") || "/echo";
                that.@com.example.echo.client.EchoClient::updateView(Ljava/lang/String;)(path);
            }
        });
    }-*/;
}
