package com.example.echo.client;

import com.google.gwt.core.client.GWT;
import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.core.client.Scheduler;
import com.google.gwt.dom.client.Element;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HTMLPanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.Widget;

public class LightboxGalleryWidget extends Composite {

    interface LightboxGalleryWidgetUiBinder extends UiBinder<Widget, LightboxGalleryWidget> {
    }

    private static LightboxGalleryWidgetUiBinder uiBinder = GWT.create(LightboxGalleryWidgetUiBinder.class);

    @UiField Image thumbnail1;
    @UiField Image thumbnail2;
    @UiField Image thumbnail3;
    @UiField HTMLPanel lightboxModal;
    @UiField Image lightboxImage;
    @UiField Button prevButton;
    @UiField Button nextButton;

    private String[] images = new String[] {
        "images/image1.jpg",
        "images/image2.jpg",
        "images/image3.jpg"
    };
    private int currentIndex = 0;

    // Store the Bootstrap modal instance
    private JavaScriptObject modalInstance;

    public LightboxGalleryWidget() {
        initWidget(uiBinder.createAndBindUi(this));

        // Defer modal initialization to ensure the DOM is ready
        Scheduler.get().scheduleDeferred(() -> {
            initModal();
        });

        // Set up click handlers for thumbnails to open the lightbox
        thumbnail1.addClickHandler(new ClickHandler() {
            @Override
            public void onClick(ClickEvent event) {
                currentIndex = 0;
                showLightbox();
            }
        });

        thumbnail2.addClickHandler(new ClickHandler() {
            @Override
            public void onClick(ClickEvent event) {
                currentIndex = 1;
                showLightbox();
            }
        });

        thumbnail3.addClickHandler(new ClickHandler() {
            @Override
            public void onClick(ClickEvent event) {
                currentIndex = 2;
                showLightbox();
            }
        });

        // Navigation buttons
        prevButton.addClickHandler(new ClickHandler() {
            @Override
            public void onClick(ClickEvent event) {
                currentIndex = (currentIndex - 1 + images.length) % images.length;
                updateLightboxImage();
            }
        });

        nextButton.addClickHandler(new ClickHandler() {
            @Override
            public void onClick(ClickEvent event) {
                currentIndex = (currentIndex + 1) % images.length;
                updateLightboxImage();
            }
        });
    }

    private void showLightbox() {
        updateLightboxImage();
        showModal();
    }

    private void updateLightboxImage() {
        lightboxImage.setUrl(images[currentIndex]);
    }

    private native JavaScriptObject initModalNative(Element element) /*-{
        if (!$wnd.jQuery) {
            console.error("jQuery is not loaded");
            return null;
        }
        if (!$wnd.bootstrap || !$wnd.bootstrap.Modal) {
            console.error("Bootstrap Modal is not loaded");
            return null;
        }
        var modalElement = $wnd.jQuery(element).find('#lightboxModal')[0];
        if (!modalElement) {
            console.error("Modal element not found");
            return null;
        }
        return new $wnd.bootstrap.Modal(modalElement, { backdrop: true, keyboard: true });
    }-*/;

    private native void showModalNative(JavaScriptObject modal) /*-{
        if (modal) {
            modal.show();
        } else {
            console.error("Modal instance is not available");
        }
    }-*/;

    private void initModal() {
        modalInstance = initModalNative(lightboxModal.getElement());
        if (modalInstance == null) {
            GWT.log("Failed to initialize Bootstrap modal");
        }
    }

    private void showModal() {
        showModalNative(modalInstance);
    }
}
